/*
 * Copyright (C), 2000-2002 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/**
 *  System independent /proc & kvm methods.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *
 *  @version \$Id$
 *
 *  @file
 */


#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "process.h"


/**
 * Reads an process dependent entry or the proc files system
 * @param buf buffer to write to
 * @param buf_size size of buffer "buf"
 * @param name name of proc service
 * @param pid pid number of the process
 * @return TRUE if succeeded otherwise FALSE.
 */
int read_proc_file(char *buf, int buf_size, char * name, int pid) {

  int fd;
  char filename[STRLEN];


  snprintf(filename, STRLEN, "/proc/%d/%s", pid, name);

  fd = open(filename, O_RDONLY);

  if ( fd < 0 ) {

    perror("proc_interface_sysdep.c: read_proc_file()");

    return FALSE;

  }

  if ( read(fd, buf, buf_size) < 0 ) {

    perror("proc_interface_sysdep.c: read_proc_file()");

    return FALSE;

  }

  close(fd);

  return TRUE;
}

/**
 * Get the actual time as a floating point number
 * @return time in seconds
 */
double get_float_time(void) {    

  struct timeval t;

  gettimeofday(&t, NULL);
  return (double) t.tv_sec * 10 + (double) t.tv_usec / 100000.0;

}

/**
 * Get the proc infomation (CPU percentage, MEM in MByte and percent,
 * status)
 * @param p proc information structure to work with.
 * @return TRUE if succeeded otherwise FALSE.
 */
int get_process_info(Proc_Info_T p) {
  int fd;
  double temp_time;

  /* get the actual time */

  temp_time = get_float_time();
  
  /* get the system dependant proc data */

  if ( ! get_process_info_sysdep(p) ) {

    return FALSE;

  }

  /* check if it is the first run */
  
  if ( p->time_prev == 0.0 ) {
    
    p->cpu_percent= 0;
    
  } else {
    
    p->cpu_percent=  (int) (( p->cputime - p->cputime_prev ) * 1000) / 
      ( temp_time - p->time_prev );
	
  }

  p->time_prev = temp_time;
    

  return TRUE;

}

